#!/bin/sh
find -name "*.cpp" -o -iname "*.cc" -o -iname "*.h" -o -iname "*.ui" -o -iname "*.kcf*" |egrep -v build | xargs grep -Pi '^(?:(?!\/\/).)*\b(KIO::NetAccess|k3bufferedsocket|k3clientsocketbase|k3resolver|k3reverseresolver|k3serversocket|k3socketaddress|k3socketbase|k3socketdevice|k3streamsocket|k4aboutdata|k4style|k4timezonewidget|kaccelgen|\bkaction\b|kapplication|karrowbutton|kascii|kbuttongroup|kcalendarsystem|kcmdlineargs|kcolorchoosermode|kcolordialog|kcolorvalueselector|kcomponentdata|kconfiggroup_kurl|kcurrencycode|kdatatool|kdatetime|kdatetimewidget|kdatewidget|kdebug|kdefakes|kde_file|kdelibs4support_export|kdelibs4support_export_internal|kdemacros|kdeprintdialog|kdeversion|kdevicelistmodel|kdialogbuttonbox|\bkdialog\b|kdirselectdialog|kdiskfreespace|keditlistbox|kfadewidgeteffect|kfiledialog|kfileitemactionplugin|kfilemetadataconfigurationwidget|kfilemetadatawidget|kfilemetainfo|kfilemetainfoitem|kfilemetainfowidget|kfileshare|kfilewriteplugin|kfontcombobox|kfontdialog|kgenericfactory|kglobal\b|kglobalsettings|khbox|khuesaturationselect|kicon\b|kimageio|kinputdialog|klibloader|klibrary|klistwidget\b|klocale|klocalizeddate|klockfile|kmd5|kmenubar|\bkmenu\b|kmessageboxmessagehandler|kmessagebox_queued|\bkmimetype\b|knuminput|knumvalidator|kpassivepopupmessagehandler|kpixmapcache|kprintpreview|kprogressdialog|kpropsdialog|kpushbutton|krestrictedline|ksavefile|kscan|ksessionmanager|ksharedptr|kshortcut\b|ksocketfactory|ksplashscreen|ksslcertchain|ksslcertificate|kssl|ksslutils|ksslx509v3|kstandarddirs|kstatusbar|kstringhandler_deprecated|kstringvalidator|ksystemtimezone|ksystemtrayicon|ktabbar|ktabwidget|ktempdir|ktemporaryfile|ktextbrowser|ktimezone|ktypelist|ktzfiletimezone|kundostack|kuniqueapplication|kurl\b|kvbox|kviewstatesaver|kxerrorhandler|predicateproperties|qtest_kde)\b'

